@@ -170,7 +170,7 @@ impl Default for Buffer {
 impl BufferRef {
     #[doc(alias = "gst_buffer_map")]
     #[inline]
-    pub fn map_readable(&self) -> Result<BufferMap<Readable>, glib::BoolError> {
+    pub fn map_readable(&self) -> Result<BufferMap<'_, Readable>, glib::BoolError> {
         unsafe {
             let mut map_info = mem::MaybeUninit::uninit();
             let res =
@@ -189,7 +189,7 @@ impl BufferRef {
 
     #[doc(alias = "gst_buffer_map")]
     #[inline]
-    pub fn map_writable(&mut self) -> Result<BufferMap<Writable>, glib::BoolError> {
+    pub fn map_writable(&mut self) -> Result<BufferMap<'_, Writable>, glib::BoolError> {
         unsafe {
             let mut map_info = mem::MaybeUninit::uninit();
             let res = ffi::gst_buffer_map(
@@ -221,7 +221,7 @@ impl BufferRef {
                 return Err(glib::bool_error!("Invalid range start"));
             }
             ops::Bound::Included(idx) => *idx,
-            ops::Bound::Excluded(idx) if idx.checked_add(1).map_or(true, |idx| idx >= n_memory) => {
+            ops::Bound::Excluded(idx) if idx.checked_add(1).is_none_or(|idx| idx >= n_memory) => {
                 return Err(glib::bool_error!("Invalid range start"));
             }
             ops::Bound::Excluded(idx) => *idx + 1,
@@ -229,7 +229,7 @@ impl BufferRef {
         };
 
         let end_idx = match range.end_bound() {
-            ops::Bound::Included(idx) if idx.checked_add(1).map_or(true, |idx| idx > n_memory) => {
+            ops::Bound::Included(idx) if idx.checked_add(1).is_none_or(|idx| idx > n_memory) => {
                 return Err(glib::bool_error!("Invalid range end"));
             }
             ops::Bound::Included(idx) => *idx + 1,
@@ -251,7 +251,7 @@ impl BufferRef {
     pub fn map_range_readable(
         &self,
         range: impl RangeBounds<usize>,
-    ) -> Result<BufferMap<Readable>, glib::BoolError> {
+    ) -> Result<BufferMap<'_, Readable>, glib::BoolError> {
         let (idx, len) = self.memory_range_into_idx_len(range)?;
         unsafe {
             let mut map_info = mem::MaybeUninit::uninit();
@@ -279,7 +279,7 @@ impl BufferRef {
     pub fn map_range_writable(
         &mut self,
         range: impl RangeBounds<usize>,
-    ) -> Result<BufferMap<Writable>, glib::BoolError> {
+    ) -> Result<BufferMap<'_, Writable>, glib::BoolError> {
         let (idx, len) = self.memory_range_into_idx_len(range)?;
         unsafe {
             let mut map_info = mem::MaybeUninit::uninit();
@@ -313,7 +313,7 @@ impl BufferRef {
                 return Err(glib::bool_error!("Invalid range start"));
             }
             ops::Bound::Included(idx) => *idx,
-            ops::Bound::Excluded(idx) if idx.checked_add(1).map_or(true, |idx| idx >= size) => {
+            ops::Bound::Excluded(idx) if idx.checked_add(1).is_none_or(|idx| idx >= size) => {
                 return Err(glib::bool_error!("Invalid range start"));
             }
             ops::Bound::Excluded(idx) => *idx + 1,
@@ -321,7 +321,7 @@ impl BufferRef {
         };
 
         let end_idx = match range.end_bound() {
-            ops::Bound::Included(idx) if idx.checked_add(1).map_or(true, |idx| idx > size) => {
+            ops::Bound::Included(idx) if idx.checked_add(1).is_none_or(|idx| idx > size) => {
                 return Err(glib::bool_error!("Invalid range end"));
             }
             ops::Bound::Included(idx) => *idx + 1,
@@ -553,7 +553,7 @@ impl BufferRef {
     #[doc(alias = "get_meta")]
     #[doc(alias = "gst_buffer_get_meta")]
     #[inline]
-    pub fn meta<T: MetaAPI>(&self) -> Option<MetaRef<T>> {
+    pub fn meta<T: MetaAPI>(&self) -> Option<MetaRef<'_, T>> {
         unsafe {
             let meta = ffi::gst_buffer_get_meta(self.as_mut_ptr(), T::meta_api().into_glib());
             if meta.is_null() {
@@ -566,7 +566,7 @@ impl BufferRef {
 
     #[doc(alias = "get_meta_mut")]
     #[inline]
-    pub fn meta_mut<T: MetaAPI>(&mut self) -> Option<MetaRefMut<T, crate::meta::Standalone>> {
+    pub fn meta_mut<T: MetaAPI>(&mut self) -> Option<MetaRefMut<'_, T, crate::meta::Standalone>> {
         unsafe {
             let meta = ffi::gst_buffer_get_meta(self.as_mut_ptr(), T::meta_api().into_glib());
             if meta.is_null() {
@@ -577,11 +577,11 @@ impl BufferRef {
         }
     }
 
-    pub fn iter_meta<T: MetaAPI>(&self) -> MetaIter<T> {
+    pub fn iter_meta<T: MetaAPI>(&self) -> MetaIter<'_, T> {
         MetaIter::new(self)
     }
 
-    pub fn iter_meta_mut<T: MetaAPI>(&mut self) -> MetaIterMut<T> {
+    pub fn iter_meta_mut<T: MetaAPI>(&mut self) -> MetaIterMut<'_, T> {
         MetaIterMut::new(self)
     }
 
@@ -826,11 +826,11 @@ impl BufferRef {
         }
     }
 
-    pub fn iter_memories(&self) -> Iter {
+    pub fn iter_memories(&self) -> Iter<'_> {
         Iter::new(self)
     }
 
-    pub fn iter_memories_mut(&mut self) -> Result<IterMut, glib::BoolError> {
+    pub fn iter_memories_mut(&mut self) -> Result<IterMut<'_>, glib::BoolError> {
         if !self.is_all_memory_writable() {
             Err(glib::bool_error!("Not all memory are writable"))
         } else {
@@ -838,7 +838,7 @@ impl BufferRef {
         }
     }
 
-    pub fn iter_memories_owned(&self) -> IterOwned {
+    pub fn iter_memories_owned(&self) -> IterOwned<'_> {
         IterOwned::new(self)
     }
 
@@ -853,7 +853,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "gst_util_dump_buffer")]
-    pub fn dump(&self) -> Dump {
+    pub fn dump(&self) -> Dump<'_> {
         Dump {
             buffer: self,
             start: Bound::Unbounded,
@@ -862,7 +862,7 @@ impl BufferRef {
     }
 
     #[doc(alias = "gst_util_dump_buffer")]
-    pub fn dump_range(&self, range: impl RangeBounds<usize>) -> Dump {
+    pub fn dump_range(&self, range: impl RangeBounds<usize>) -> Dump<'_> {
         Dump {
             buffer: self,
             start: range.start_bound().cloned(),
@@ -873,6 +873,7 @@ impl BufferRef {
 
 macro_rules! define_meta_iter(
     ($name:ident, $typ:ty, $mtyp:ty, $prepare_buffer:expr, $from_ptr:expr) => {
+    #[must_use = "iterators are lazy and do nothing unless consumed"]
     pub struct $name<'a, T: MetaAPI + 'a> {
         buffer: $typ,
         state: glib::ffi::gpointer,
@@ -950,121 +951,11 @@ define_meta_iter!(
 
 macro_rules! define_iter(
     ($name:ident, $typ:ty, $mtyp:ty, $get_item:expr) => {
-    pub struct $name<'a> {
-        buffer: $typ,
-        idx: usize,
-        n_memory: usize,
-    }
-
-    impl<'a> fmt::Debug for $name<'a> {
-        fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-            f.debug_struct(stringify!($name))
-                .field("buffer", &self.buffer)
-                .field("idx", &self.idx)
-                .field("n_memory", &self.n_memory)
-                .finish()
-        }
-    }
-
-    impl<'a> $name<'a> {
-        fn new(buffer: $typ) -> $name<'a> {
-            skip_assert_initialized!();
-
-            let n_memory = buffer.n_memory();
-
-            $name {
-                buffer,
-                idx: 0,
-                n_memory,
-            }
-        }
-    }
-
-    #[allow(clippy::redundant_closure_call)]
-    impl<'a> Iterator for $name<'a> {
-        type Item = $mtyp;
-
-        fn next(&mut self) -> Option<Self::Item> {
-            if self.idx >= self.n_memory {
-                return None;
-            }
-
-            #[allow(unused_unsafe)]
-            unsafe {
-                let item = $get_item(self.buffer, self.idx).unwrap();
-                self.idx += 1;
-                Some(item)
-            }
-        }
-
-        fn size_hint(&self) -> (usize, Option<usize>) {
-            let remaining = self.n_memory - self.idx;
-
-            (remaining, Some(remaining))
-        }
-
-        fn count(self) -> usize {
-            self.n_memory - self.idx
-        }
-
-        fn nth(&mut self, n: usize) -> Option<Self::Item> {
-            let (end, overflow) = self.idx.overflowing_add(n);
-            if end >= self.n_memory || overflow {
-                self.idx = self.n_memory;
-                None
-            } else {
-                #[allow(unused_unsafe)]
-                unsafe {
-                    self.idx = end + 1;
-                    Some($get_item(self.buffer, end).unwrap())
-                }
-            }
-        }
-
-        fn last(self) -> Option<Self::Item> {
-            if self.idx == self.n_memory {
-                None
-            } else {
-                #[allow(unused_unsafe)]
-                unsafe {
-                    Some($get_item(self.buffer, self.n_memory - 1).unwrap())
-                }
-            }
-        }
-    }
-
-    #[allow(clippy::redundant_closure_call)]
-    impl<'a> DoubleEndedIterator for $name<'a> {
-        fn next_back(&mut self) -> Option<Self::Item> {
-            if self.idx == self.n_memory {
-                return None;
-            }
-
-            #[allow(unused_unsafe)]
-            unsafe {
-                self.n_memory -= 1;
-                Some($get_item(self.buffer, self.n_memory).unwrap())
-            }
-        }
-
-        fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
-            let (end, overflow) = self.n_memory.overflowing_sub(n);
-            if end <= self.idx || overflow {
-                self.idx = self.n_memory;
-                None
-            } else {
-                #[allow(unused_unsafe)]
-                unsafe {
-                    self.n_memory = end - 1;
-                    Some($get_item(self.buffer, self.n_memory).unwrap())
-                }
-            }
-        }
-    }
-
-    impl<'a> ExactSizeIterator for $name<'a> {}
-
-    impl<'a> std::iter::FusedIterator for $name<'a> {}
+        crate::utils::define_fixed_size_iter!(
+            $name, $typ, $mtyp,
+            |buffer: &BufferRef| buffer.n_memory() as usize,
+            $get_item
+        );
     }
 );
 
@@ -1072,13 +963,9 @@ define_iter!(
     Iter,
     &'a BufferRef,
     &'a MemoryRef,
-    |buffer: &BufferRef, idx| {
+    |buffer: &BufferRef, idx| unsafe {
         let ptr = ffi::gst_buffer_peek_memory(buffer.as_mut_ptr(), idx as u32);
-        if ptr.is_null() {
-            None
-        } else {
-            Some(MemoryRef::from_ptr(ptr as *const ffi::GstMemory))
-        }
+        MemoryRef::from_ptr(ptr as *const ffi::GstMemory)
     }
 );
 
@@ -1086,13 +973,9 @@ define_iter!(
     IterMut,
     &'a mut BufferRef,
     &'a mut MemoryRef,
-    |buffer: &mut BufferRef, idx| {
+    |buffer: &mut BufferRef, idx| unsafe {
         let ptr = ffi::gst_buffer_peek_memory(buffer.as_mut_ptr(), idx as u32);
-        if ptr.is_null() {
-            None
-        } else {
-            Some(MemoryRef::from_mut_ptr(ptr))
-        }
+        MemoryRef::from_mut_ptr(ptr)
     }
 );
 
@@ -1157,7 +1040,10 @@ define_iter!(
     IterOwned,
     &'a BufferRef,
     Memory,
-    |buffer: &BufferRef, idx| { buffer.memory(idx) }
+    |buffer: &BufferRef, idx| unsafe {
+        let ptr = ffi::gst_buffer_get_memory(buffer.as_mut_ptr(), idx as u32);
+        from_glib_full(ptr)
+    }
 );
 
 impl fmt::Debug for Buffer {
@@ -1446,6 +1332,7 @@ pub struct Dump<'a> {
     end: Bound<usize>,
 }
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 struct BufferChunked16Iter<'a> {
     buffer: &'a BufferRef,
     mem_idx: usize,
@@ -1512,7 +1399,7 @@ impl Dump<'_> {
                 write!(f, "<start out of range>")?;
                 return Ok(());
             }
-            Bound::Excluded(idx) if idx.checked_add(1).map_or(true, |idx| idx >= len) => {
+            Bound::Excluded(idx) if idx.checked_add(1).is_none_or(|idx| idx >= len) => {
                 write!(f, "<start out of range>")?;
                 return Ok(());
             }
@@ -1522,7 +1409,7 @@ impl Dump<'_> {
         };
 
         let end_idx = match self.end {
-            Bound::Included(idx) if idx.checked_add(1).map_or(true, |idx| idx > len) => {
+            Bound::Included(idx) if idx.checked_add(1).is_none_or(|idx| idx > len) => {
                 write!(f, "<end out of range>")?;
                 return Ok(());
             }
@@ -1560,17 +1447,17 @@ impl Dump<'_> {
                 let line = &line[..line_len];
 
                 match end_idx {
-                    0x00_00..=0xff_ff => write!(f, "{:04x}:  ", start_idx)?,
-                    0x01_00_00..=0xff_ff_ff => write!(f, "{:06x}:  ", start_idx)?,
-                    0x01_00_00_00..=0xff_ff_ff_ff => write!(f, "{:08x}:  ", start_idx)?,
-                    _ => write!(f, "{:016x}:  ", start_idx)?,
+                    0x00_00..=0xff_ff => write!(f, "{start_idx:04x}:  ")?,
+                    0x01_00_00..=0xff_ff_ff => write!(f, "{start_idx:06x}:  ")?,
+                    0x01_00_00_00..=0xff_ff_ff_ff => write!(f, "{start_idx:08x}:  ")?,
+                    _ => write!(f, "{start_idx:016x}:  ")?,
                 }
 
                 for (i, v) in line.iter().enumerate() {
                     if i > 0 {
-                        write!(f, " {:02x}", v)?;
+                        write!(f, " {v:02x}")?;
                     } else {
-                        write!(f, "{:02x}", v)?;
+                        write!(f, "{v:02x}")?;
                     }
                 }
 
@@ -1600,9 +1487,9 @@ impl Dump<'_> {
 
                 for (i, v) in line.iter().enumerate() {
                     if i > 0 {
-                        write!(f, " {:02x}", v)?;
+                        write!(f, " {v:02x}")?;
                     } else {
-                        write!(f, "{:02x}", v)?;
+                        write!(f, "{v:02x}")?;
                     }
                 }
 
