@@ -2,9 +2,23 @@
 
 use glib::{prelude::*, translate::*};
 
-use crate::{ClockTime, ControlBinding};
+use crate::{ffi, ClockTime, ControlBinding};
 
 pub trait ControlBindingExtManual: IsA<ControlBinding> + 'static {
+    fn property_name(&self) -> &glib::GStr {
+        unsafe {
+            let ptr: *mut ffi::GstControlBinding = self.as_ref().to_glib_none().0;
+            glib::GStr::from_ptr((*ptr).name)
+        }
+    }
+
+    fn property_spec(&self) -> &glib::ParamSpec {
+        unsafe {
+            let ptr: *mut ffi::GstControlBinding = self.as_ref().to_glib_none().0;
+            glib::ParamSpec::from_glib_ptr_borrow(&(*ptr).pspec)
+        }
+    }
+
     #[doc(alias = "get_g_value_array")]
     #[doc(alias = "gst_control_binding_get_g_value_array")]
     fn g_value_array(
