@@ -5,9 +5,9 @@ use std::ptr;
 use glib::{bool_error, prelude::*, subclass::prelude::*, translate::*, BoolError};
 
 use super::prelude::*;
-use crate::{AllocationParams, Allocator, Memory};
+use crate::{ffi, AllocationParams, Allocator, Memory};
 
-pub trait AllocatorImpl: AllocatorImplExt + GstObjectImpl + Send + Sync {
+pub trait AllocatorImpl: GstObjectImpl + ObjectSubclass<Type: IsA<Allocator>> {
     fn alloc(&self, size: usize, params: Option<&AllocationParams>) -> Result<Memory, BoolError> {
         self.parent_alloc(size, params)
     }
@@ -17,12 +17,7 @@ pub trait AllocatorImpl: AllocatorImplExt + GstObjectImpl + Send + Sync {
     }
 }
 
-mod sealed {
-    pub trait Sealed {}
-    impl<T: super::AllocatorImplExt> Sealed for T {}
-}
-
-pub trait AllocatorImplExt: sealed::Sealed + ObjectSubclass {
+pub trait AllocatorImplExt: AllocatorImpl {
     fn parent_alloc(
         &self,
         size: usize,
@@ -89,7 +84,7 @@ unsafe extern "C" fn alloc<T: AllocatorImpl>(
     imp.alloc(size, params)
         .map(|memory| memory.into_glib_ptr())
         .unwrap_or_else(|error| {
-            error!(crate::CAT_RUST, obj: instance, "{:?}", error);
+            error!(crate::CAT_RUST, obj = instance, "{:?}", error);
 
             ptr::null_mut()
         })
