@@ -1,26 +1,19 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::ffi::CStr;
-use std::fmt;
-use std::marker::PhantomData;
-use std::mem;
+use std::{fmt, marker::PhantomData, mem};
 
-use once_cell::sync::Lazy;
-
-use glib::translate::{
-    from_glib, from_glib_full, FromGlibPtrFull, IntoGlib, ToGlibPtr, ToGlibPtrMut,
+use glib::{
+    prelude::*,
+    translate::*,
+    value::{FromValue, SendValue, ToSendValue, Value},
+    GStr,
 };
-use glib::value::{FromValue, SendValue, ToSendValue, Value};
-use glib::StaticType;
 
-use crate::Sample;
-use crate::TagError;
-use crate::TagMergeMode;
-use crate::TagScope;
+use crate::{ffi, Sample, TagError, TagMergeMode, TagScope};
 
 pub trait Tag<'a> {
     type TagType: StaticType + FromValue<'a> + ToSendValue + Send + Sync;
-    fn tag_name<'b>() -> &'b str;
+    const TAG_NAME: &'static glib::GStr;
 }
 
 macro_rules! impl_tag(
@@ -28,14 +21,8 @@ macro_rules! impl_tag(
         pub enum $name {}
         impl<'a> Tag<'a> for $name {
             type TagType = $t;
-
-            fn tag_name<'b>() -> &'b str {
-                *$rust_tag
-            }
+            const TAG_NAME: &'static glib::GStr = unsafe { glib::GStr::from_utf8_with_nul_unchecked(ffi::$gst_tag) };
         }
-
-        pub(crate) static $rust_tag: Lazy<&'static str> = Lazy::new(||
-            unsafe { CStr::from_ptr(ffi::$gst_tag).to_str().unwrap() });
     };
 );
 
@@ -215,7 +202,7 @@ impl_tag!(
     GST_TAG_GEO_LOCATION_LATITUDE
 );
 impl_tag!(
-    GeoLocationLongitute,
+    GeoLocationLongitude,
     f64,
     TAG_GEO_LOCATION_LONGITUDE,
     GST_TAG_GEO_LOCATION_LONGITUDE
@@ -336,6 +323,23 @@ impl_tag!(
 );
 impl_tag!(PrivateData, Sample, TAG_PRIVATE_DATA, GST_TAG_PRIVATE_DATA);
 
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+mod v1_24 {
+    use super::*;
+
+    impl_tag!(
+        ContainerSpecificTrackId,
+        &'a str,
+        TAG_CONTAINER_SPECIFIC_TRACK_ID,
+        GST_TAG_CONTAINER_SPECIFIC_TRACK_ID
+    );
+}
+
+#[cfg(feature = "v1_24")]
+#[cfg_attr(docsrs, doc(cfg(feature = "v1_24")))]
+pub use v1_24::ContainerSpecificTrackId;
+
 mini_object_wrapper!(TagList, TagListRef, ffi::GstTagList, || {
     ffi::gst_tag_list_get_type()
 });
@@ -370,69 +374,67 @@ impl<T> TagValue<T> {
 impl TagListRef {
     #[doc(alias = "gst_tag_list_add")]
     pub fn add<'a, T: Tag<'a>>(&mut self, value: &T::TagType, mode: TagMergeMode) {
-        // result can be safely ignored here as `value`'s type is tied to `T::tag_name()`
+        // result can be safely ignored here as `value`'s type is tied to `T::TAG_NAME`
         let v = <T::TagType as ToSendValue>::to_send_value(value);
-        let _res = self.add_value(T::tag_name(), &v, mode);
+        let _res = self.add_value(T::TAG_NAME, &v, mode);
     }
 
     #[doc(alias = "gst_tag_list_add")]
-    pub fn add_generic<T: ToSendValue + Sync>(
+    pub fn add_generic(
         &mut self,
-        tag_name: &str,
-        value: T,
+        tag_name: impl IntoGStr,
+        value: impl ToSendValue,
         mode: TagMergeMode,
     ) -> Result<(), TagError> {
-        let v = value.to_send_value();
-        self.add_value(tag_name, &v, mode)
+        self.add_value(tag_name, &value.to_send_value(), mode)
     }
 
     #[doc(alias = "gst_tag_list_add_value")]
     pub fn add_value(
         &mut self,
-        tag_name: &str,
+        tag_name: impl IntoGStr,
         value: &glib::SendValue,
         mode: TagMergeMode,
     ) -> Result<(), TagError> {
         unsafe {
-            let tag_name = tag_name.to_glib_none();
+            tag_name.run_with_gstr(|tag_name| {
+                let tag_type: glib::Type = from_glib(ffi::gst_tag_get_type(tag_name.as_ptr()));
+                if tag_type != value.type_() {
+                    return Err(TagError::TypeMismatch);
+                }
 
-            let tag_type: glib::Type = from_glib(ffi::gst_tag_get_type(tag_name.0));
-            if tag_type != value.type_() {
-                return Err(TagError::TypeMismatch);
-            }
-
-            ffi::gst_tag_list_add_value(
-                self.as_mut_ptr(),
-                mode.into_glib(),
-                tag_name.0,
-                value.to_glib_none().0,
-            );
+                ffi::gst_tag_list_add_value(
+                    self.as_mut_ptr(),
+                    mode.into_glib(),
+                    tag_name.as_ptr(),
+                    value.to_glib_none().0,
+                );
+                Ok(())
+            })
         }
-
-        Ok(())
     }
 
     #[doc(alias = "gst_tag_list_remove_tag")]
     pub fn remove<'a, T: Tag<'a>>(&mut self) {
-        self.remove_generic(T::tag_name());
+        self.remove_generic(T::TAG_NAME);
     }
 
     #[doc(alias = "gst_tag_list_remove_tag")]
-    pub fn remove_generic(&mut self, tag_name: &str) {
+    pub fn remove_generic(&mut self, tag_name: impl IntoGStr) {
         unsafe {
-            let tag_name = tag_name.to_glib_none();
-
-            ffi::gst_tag_list_remove_tag(self.as_mut_ptr(), tag_name.0);
+            tag_name.run_with_gstr(|tag_name| {
+                ffi::gst_tag_list_remove_tag(self.as_mut_ptr(), tag_name.as_ptr());
+            })
         }
     }
 
     #[doc(alias = "gst_tag_list_get")]
     pub fn get<'a, T: Tag<'a>>(&self) -> Option<TagValue<T::TagType>> {
-        self.generic(T::tag_name()).map(|value| {
+        self.generic(T::TAG_NAME).map(|value| {
             if !value.is::<T::TagType>() {
                 panic!(
                     "TagListRef::get type mismatch for tag {}: {}",
-                    T::tag_name(),
+                    T::TAG_NAME,
                     value.type_()
                 );
             }
@@ -442,50 +444,52 @@ impl TagListRef {
 
     #[doc(alias = "gst_tag_list_get")]
     #[doc(alias = "get_generic")]
-    pub fn generic(&self, tag_name: &str) -> Option<SendValue> {
+    pub fn generic(&self, tag_name: impl IntoGStr) -> Option<SendValue> {
         unsafe {
             let mut value: mem::MaybeUninit<SendValue> = mem::MaybeUninit::zeroed();
 
-            let found: bool = from_glib(ffi::gst_tag_list_copy_value(
-                (*value.as_mut_ptr()).to_glib_none_mut().0,
-                self.as_ptr(),
-                tag_name.to_glib_none().0,
-            ));
+            let found: bool = tag_name.run_with_gstr(|tag_name| {
+                from_glib(ffi::gst_tag_list_copy_value(
+                    (*value.as_mut_ptr()).to_glib_none_mut().0,
+                    self.as_ptr(),
+                    tag_name.as_ptr(),
+                ))
+            });
 
             if !found {
-                return None;
+                None
+            } else {
+                Some(value.assume_init())
             }
-
-            Some(value.assume_init())
         }
     }
 
     #[doc(alias = "gst_tag_list_n_tags")]
-    pub fn n_tags(&self) -> u32 {
-        unsafe { ffi::gst_tag_list_n_tags(self.as_ptr()) as u32 }
+    pub fn n_tags(&self) -> usize {
+        unsafe { ffi::gst_tag_list_n_tags(self.as_ptr()) as usize }
     }
 
     #[doc(alias = "gst_tag_list_nth_tag_name")]
-    pub fn nth_tag_name(&self, idx: u32) -> Option<&str> {
+    pub fn nth_tag_name(&self, idx: usize) -> Option<&glib::GStr> {
         if idx >= self.n_tags() {
             return None;
         }
 
         unsafe {
-            let name = ffi::gst_tag_list_nth_tag_name(self.as_ptr(), idx);
-            assert!(!name.is_null());
-            Some(CStr::from_ptr(name).to_str().unwrap())
+            let name = ffi::gst_tag_list_nth_tag_name(self.as_ptr(), idx as u32);
+            debug_assert!(!name.is_null());
+            Some(glib::GStr::from_ptr(name))
         }
     }
 
     #[doc(alias = "get_index")]
     #[doc(alias = "gst_tag_list_get_index")]
-    pub fn index<'a, T: Tag<'a>>(&self, idx: u32) -> Option<&'a TagValue<T::TagType>> {
-        self.index_generic(T::tag_name(), idx).map(|value| {
+    pub fn index<'a, T: Tag<'a>>(&'a self, idx: usize) -> Option<&'a TagValue<T::TagType>> {
+        self.index_generic(T::TAG_NAME, idx).map(|value| {
             if !value.is::<T::TagType>() {
                 panic!(
                     "TagListRef::get_index type mismatch for tag {}: {}",
-                    T::tag_name(),
+                    T::TAG_NAME,
                     value.type_()
                 );
             }
@@ -495,44 +499,50 @@ impl TagListRef {
 
     #[doc(alias = "get_index_generic")]
     #[doc(alias = "gst_tag_list_get_index")]
-    pub fn index_generic<'a>(&'a self, tag_name: &str, idx: u32) -> Option<&'a SendValue> {
+    pub fn index_generic(&self, tag_name: impl IntoGStr, idx: usize) -> Option<&SendValue> {
         unsafe {
-            let value =
-                ffi::gst_tag_list_get_value_index(self.as_ptr(), tag_name.to_glib_none().0, idx);
+            let idx = u32::try_from(idx).ok()?;
+            let value = tag_name.run_with_gstr(|tag_name| {
+                ffi::gst_tag_list_get_value_index(self.as_ptr(), tag_name.as_ptr(), idx)
+            });
 
             if value.is_null() {
-                return None;
+                None
+            } else {
+                Some(&*(value as *const SendValue))
             }
-
-            Some(&*(value as *const SendValue))
         }
     }
 
     #[doc(alias = "get_size")]
     #[doc(alias = "gst_tag_list_get_tag_size")]
-    pub fn size<'a, T: Tag<'a>>(&self) -> u32 {
-        self.size_by_name(T::tag_name())
+    pub fn size<'a, T: Tag<'a>>(&self) -> usize {
+        self.size_by_name(T::TAG_NAME)
     }
 
     #[doc(alias = "get_size_by_name")]
     #[doc(alias = "gst_tag_list_get_tag_size")]
-    pub fn size_by_name(&self, tag_name: &str) -> u32 {
-        unsafe { ffi::gst_tag_list_get_tag_size(self.as_ptr(), tag_name.to_glib_none().0) }
+    pub fn size_by_name(&self, tag_name: impl IntoGStr) -> usize {
+        unsafe {
+            tag_name.run_with_gstr(|tag_name| {
+                ffi::gst_tag_list_get_tag_size(self.as_ptr(), tag_name.as_ptr()) as usize
+            })
+        }
     }
 
     pub fn iter_tag<'a, T: Tag<'a>>(&'a self) -> TagIter<'a, T> {
         TagIter::new(self)
     }
 
-    pub fn iter_tag_generic<'a>(&'a self, tag_name: &'a str) -> GenericTagIter<'a> {
+    pub fn iter_tag_generic<'a>(&'a self, tag_name: &'a GStr) -> GenericTagIter<'a> {
         GenericTagIter::new(self, tag_name)
     }
 
-    pub fn iter_generic(&self) -> GenericIter {
+    pub fn iter_generic(&self) -> GenericIter<'_> {
         GenericIter::new(self)
     }
 
-    pub fn iter(&self) -> Iter {
+    pub fn iter(&self) -> Iter<'_> {
         Iter::new(self)
     }
 
@@ -625,6 +635,7 @@ impl PartialEq for TagListRef {
 
 impl Eq for TagListRef {}
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 #[derive(Debug)]
 pub struct TagIter<'a, T: Tag<'a>> {
     taglist: &'a TagListRef,
@@ -639,7 +650,7 @@ impl<'a, T: Tag<'a>> TagIter<'a, T> {
         TagIter {
             taglist,
             idx: 0,
-            size: taglist.size::<T>() as usize,
+            size: taglist.size::<T>(),
             phantom: PhantomData,
         }
     }
@@ -657,7 +668,7 @@ where
             return None;
         }
 
-        let item = self.taglist.index::<T>(self.idx as u32).unwrap();
+        let item = self.taglist.index::<T>(self.idx).unwrap();
         self.idx += 1;
 
         Some(item)
@@ -680,7 +691,7 @@ where
             None
         } else {
             self.idx = end + 1;
-            Some(self.taglist.index::<T>(end as u32).unwrap())
+            Some(self.taglist.index::<T>(end).unwrap())
         }
     }
 
@@ -688,7 +699,7 @@ where
         if self.idx == self.size {
             None
         } else {
-            Some(self.taglist.index::<T>(self.size as u32 - 1).unwrap())
+            Some(self.taglist.index::<T>(self.size - 1).unwrap())
         }
     }
 }
@@ -704,7 +715,7 @@ where
         }
 
         self.size -= 1;
-        Some(self.taglist.index::<T>(self.size as u32).unwrap())
+        Some(self.taglist.index::<T>(self.size).unwrap())
     }
 
     fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
@@ -714,7 +725,7 @@ where
             None
         } else {
             self.size = end - 1;
-            Some(self.taglist.index::<T>(self.size as u32).unwrap())
+            Some(self.taglist.index::<T>(self.size).unwrap())
         }
     }
 }
@@ -733,22 +744,23 @@ where
 {
 }
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 #[derive(Debug)]
 pub struct GenericTagIter<'a> {
     taglist: &'a TagListRef,
-    name: &'a str,
+    name: &'a GStr,
     idx: usize,
     size: usize,
 }
 
 impl<'a> GenericTagIter<'a> {
-    fn new(taglist: &'a TagListRef, name: &'a str) -> GenericTagIter<'a> {
+    fn new(taglist: &'a TagListRef, name: &'a GStr) -> GenericTagIter<'a> {
         skip_assert_initialized!();
         GenericTagIter {
             taglist,
             name,
             idx: 0,
-            size: taglist.size_by_name(name) as usize,
+            size: taglist.size_by_name(name),
         }
     }
 }
@@ -761,10 +773,7 @@ impl<'a> Iterator for GenericTagIter<'a> {
             return None;
         }
 
-        let item = self
-            .taglist
-            .index_generic(self.name, self.idx as u32)
-            .unwrap();
+        let item = self.taglist.index_generic(self.name, self.idx).unwrap();
         self.idx += 1;
 
         Some(item)
@@ -787,7 +796,7 @@ impl<'a> Iterator for GenericTagIter<'a> {
             None
         } else {
             self.idx = end + 1;
-            Some(self.taglist.index_generic(self.name, end as u32).unwrap())
+            Some(self.taglist.index_generic(self.name, end).unwrap())
         }
     }
 
@@ -797,25 +806,21 @@ impl<'a> Iterator for GenericTagIter<'a> {
         } else {
             Some(
                 self.taglist
-                    .index_generic(self.name, self.size as u32 - 1)
+                    .index_generic(self.name, self.size - 1)
                     .unwrap(),
             )
         }
     }
 }
 
-impl<'a> DoubleEndedIterator for GenericTagIter<'a> {
+impl DoubleEndedIterator for GenericTagIter<'_> {
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.idx == self.size {
             return None;
         }
 
         self.size -= 1;
-        Some(
-            self.taglist
-                .index_generic(self.name, self.size as u32)
-                .unwrap(),
-        )
+        Some(self.taglist.index_generic(self.name, self.size).unwrap())
     }
 
     fn nth_back(&mut self, n: usize) -> Option<Self::Item> {
@@ -825,19 +830,16 @@ impl<'a> DoubleEndedIterator for GenericTagIter<'a> {
             None
         } else {
             self.size = end - 1;
-            Some(
-                self.taglist
-                    .index_generic(self.name, self.size as u32)
-                    .unwrap(),
-            )
+            Some(self.taglist.index_generic(self.name, self.size).unwrap())
         }
     }
 }
 
-impl<'a> ExactSizeIterator for GenericTagIter<'a> {}
+impl ExactSizeIterator for GenericTagIter<'_> {}
 
-impl<'a> std::iter::FusedIterator for GenericTagIter<'a> {}
+impl std::iter::FusedIterator for GenericTagIter<'_> {}
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 #[derive(Debug)]
 pub struct GenericIter<'a> {
     taglist: &'a TagListRef,
@@ -852,20 +854,20 @@ impl<'a> GenericIter<'a> {
         GenericIter {
             taglist,
             idx: 0,
-            size: if size > 0 { size as usize } else { 0 },
+            size: if size > 0 { size } else { 0 },
         }
     }
 }
 
 impl<'a> Iterator for GenericIter<'a> {
-    type Item = (&'a str, GenericTagIter<'a>);
+    type Item = (&'a glib::GStr, GenericTagIter<'a>);
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.idx >= self.size {
             return None;
         }
 
-        let name = self.taglist.nth_tag_name(self.idx as u32).unwrap();
+        let name = self.taglist.nth_tag_name(self.idx).unwrap();
         let item = (name, self.taglist.iter_tag_generic(name));
         self.idx += 1;
 
@@ -889,7 +891,7 @@ impl<'a> Iterator for GenericIter<'a> {
             None
         } else {
             self.idx = end + 1;
-            let name = self.taglist.nth_tag_name(end as u32).unwrap();
+            let name = self.taglist.nth_tag_name(end).unwrap();
             Some((name, self.taglist.iter_tag_generic(name)))
         }
     }
@@ -898,20 +900,20 @@ impl<'a> Iterator for GenericIter<'a> {
         if self.idx == self.size {
             None
         } else {
-            let name = self.taglist.nth_tag_name(self.size as u32 - 1).unwrap();
+            let name = self.taglist.nth_tag_name(self.size - 1).unwrap();
             Some((name, self.taglist.iter_tag_generic(name)))
         }
     }
 }
 
-impl<'a> DoubleEndedIterator for GenericIter<'a> {
+impl DoubleEndedIterator for GenericIter<'_> {
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.idx == self.size {
             return None;
         }
 
         self.size -= 1;
-        let name = self.taglist.nth_tag_name(self.idx as u32).unwrap();
+        let name = self.taglist.nth_tag_name(self.idx).unwrap();
         Some((name, self.taglist.iter_tag_generic(name)))
     }
 
@@ -922,16 +924,17 @@ impl<'a> DoubleEndedIterator for GenericIter<'a> {
             None
         } else {
             self.size = end - 1;
-            let name = self.taglist.nth_tag_name(self.size as u32).unwrap();
+            let name = self.taglist.nth_tag_name(self.size).unwrap();
             Some((name, self.taglist.iter_tag_generic(name)))
         }
     }
 }
 
-impl<'a> ExactSizeIterator for GenericIter<'a> {}
+impl ExactSizeIterator for GenericIter<'_> {}
 
-impl<'a> std::iter::FusedIterator for GenericIter<'a> {}
+impl std::iter::FusedIterator for GenericIter<'_> {}
 
+#[must_use = "iterators are lazy and do nothing unless consumed"]
 #[derive(Debug)]
 pub struct Iter<'a> {
     taglist: &'a TagListRef,
@@ -946,20 +949,20 @@ impl<'a> Iter<'a> {
         Iter {
             taglist,
             idx: 0,
-            size: if size > 0 { size as usize } else { 0 },
+            size: if size > 0 { size } else { 0 },
         }
     }
 }
 
 impl<'a> Iterator for Iter<'a> {
-    type Item = (&'a str, glib::SendValue);
+    type Item = (&'a glib::GStr, glib::SendValue);
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.idx >= self.size {
             return None;
         }
 
-        let name = self.taglist.nth_tag_name(self.idx as u32).unwrap();
+        let name = self.taglist.nth_tag_name(self.idx).unwrap();
         let item = (name, self.taglist.generic(name).unwrap());
         self.idx += 1;
 
@@ -983,7 +986,7 @@ impl<'a> Iterator for Iter<'a> {
             None
         } else {
             self.idx = end + 1;
-            let name = self.taglist.nth_tag_name(end as u32).unwrap();
+            let name = self.taglist.nth_tag_name(end).unwrap();
             Some((name, self.taglist.generic(name).unwrap()))
         }
     }
@@ -992,20 +995,20 @@ impl<'a> Iterator for Iter<'a> {
         if self.idx == self.size {
             None
         } else {
-            let name = self.taglist.nth_tag_name(self.size as u32 - 1).unwrap();
+            let name = self.taglist.nth_tag_name(self.size - 1).unwrap();
             Some((name, self.taglist.generic(name).unwrap()))
         }
     }
 }
 
-impl<'a> DoubleEndedIterator for Iter<'a> {
+impl DoubleEndedIterator for Iter<'_> {
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.idx == self.size {
             return None;
         }
 
         self.size -= 1;
-        let name = self.taglist.nth_tag_name(self.idx as u32).unwrap();
+        let name = self.taglist.nth_tag_name(self.idx).unwrap();
         Some((name, self.taglist.generic(name).unwrap()))
     }
 
@@ -1016,61 +1019,61 @@ impl<'a> DoubleEndedIterator for Iter<'a> {
             None
         } else {
             self.size = end - 1;
-            let name = self.taglist.nth_tag_name(self.size as u32).unwrap();
+            let name = self.taglist.nth_tag_name(self.size).unwrap();
             Some((name, self.taglist.generic(name).unwrap()))
         }
     }
 }
 
-impl<'a> ExactSizeIterator for Iter<'a> {}
+impl ExactSizeIterator for Iter<'_> {}
 
-impl<'a> std::iter::FusedIterator for Iter<'a> {}
+impl std::iter::FusedIterator for Iter<'_> {}
 
 #[doc(alias = "gst_tag_exists")]
-pub fn tag_exists(name: &str) -> bool {
+pub fn tag_exists(name: impl IntoGStr) -> bool {
     skip_assert_initialized!();
-    unsafe { from_glib(ffi::gst_tag_exists(name.to_glib_none().0)) }
+    unsafe { name.run_with_gstr(|name| from_glib(ffi::gst_tag_exists(name.as_ptr()))) }
 }
 
 #[doc(alias = "gst_tag_get_type")]
-pub fn tag_get_type(name: &str) -> glib::Type {
+pub fn tag_get_type(name: impl IntoGStr) -> glib::Type {
     skip_assert_initialized!();
-    unsafe { from_glib(ffi::gst_tag_get_type(name.to_glib_none().0)) }
+    unsafe { name.run_with_gstr(|name| from_glib(ffi::gst_tag_get_type(name.as_ptr()))) }
 }
 
 #[doc(alias = "gst_tag_get_nick")]
-pub fn tag_get_nick(name: &str) -> &str {
+pub fn tag_get_nick<'b>(name: impl IntoGStr) -> &'b glib::GStr {
     skip_assert_initialized!();
     unsafe {
-        let ptr = ffi::gst_tag_get_nick(name.to_glib_none().0);
-        CStr::from_ptr(ptr).to_str().unwrap()
+        let ptr = name.run_with_gstr(|name| ffi::gst_tag_get_nick(name.as_ptr()));
+        glib::GStr::from_ptr(ptr)
     }
 }
 
 #[doc(alias = "gst_tag_get_description")]
-pub fn tag_get_description<'b>(name: &str) -> Option<&'b str> {
+pub fn tag_get_description<'b>(name: impl IntoGStr) -> Option<&'b glib::GStr> {
     skip_assert_initialized!();
     unsafe {
-        let ptr = ffi::gst_tag_get_description(name.to_glib_none().0);
+        let ptr = name.run_with_gstr(|name| ffi::gst_tag_get_description(name.as_ptr()));
 
         if ptr.is_null() {
             None
         } else {
-            Some(CStr::from_ptr(ptr).to_str().unwrap())
+            Some(glib::GStr::from_ptr(ptr))
         }
     }
 }
 
 #[doc(alias = "gst_tag_get_flag")]
-pub fn tag_get_flag(name: &str) -> crate::TagFlag {
+pub fn tag_get_flag(name: impl IntoGStr) -> crate::TagFlag {
     skip_assert_initialized!();
-    unsafe { from_glib(ffi::gst_tag_get_flag(name.to_glib_none().0)) }
+    unsafe { name.run_with_gstr(|name| from_glib(ffi::gst_tag_get_flag(name.as_ptr()))) }
 }
 
 pub trait CustomTag<'a>: Tag<'a> {
     const FLAG: crate::TagFlag;
-    const NICK: &'static str;
-    const DESCRIPTION: &'static str;
+    const NICK: &'static glib::GStr;
+    const DESCRIPTION: &'static glib::GStr;
 
     fn merge_func(src: &Value) -> Value {
         skip_assert_initialized!();
@@ -1080,7 +1083,7 @@ pub trait CustomTag<'a>: Tag<'a> {
 
 #[doc(alias = "gst_tag_register")]
 pub fn register<T: for<'a> CustomTag<'a>>() {
-    assert!(!tag_exists(T::tag_name()));
+    assert!(!tag_exists(T::TAG_NAME));
 
     unsafe extern "C" fn merge_func_trampoline<T: for<'a> CustomTag<'a>>(
         dest: *mut glib::gobject_ffi::GValue,
@@ -1091,11 +1094,11 @@ pub fn register<T: for<'a> CustomTag<'a>>() {
 
     unsafe {
         ffi::gst_tag_register(
-            T::tag_name().to_glib_none().0,
+            T::TAG_NAME.as_ptr(),
             T::FLAG.into_glib(),
             T::TagType::static_type().into_glib(),
-            T::NICK.to_glib_none().0,
-            T::DESCRIPTION.to_glib_none().0,
+            T::NICK.as_ptr(),
+            T::DESCRIPTION.as_ptr(),
             Some(merge_func_trampoline::<T>),
         )
     }
@@ -1107,8 +1110,6 @@ pub fn merge_use_first(src: &Value) -> Value {
     assert_eq!(src.type_(), crate::List::static_type());
 
     unsafe {
-        use glib::translate::Uninitialized;
-
         let mut res = Value::uninitialized();
         ffi::gst_tag_merge_use_first(res.to_glib_none_mut().0, src.to_glib_none().0);
         res
@@ -1121,8 +1122,6 @@ pub fn merge_strings_with_comma(src: &Value) -> Value {
     assert_eq!(src.type_(), crate::List::static_type());
 
     unsafe {
-        use glib::translate::Uninitialized;
-
         let mut res = Value::uninitialized();
         ffi::gst_tag_merge_strings_with_comma(res.to_glib_none_mut().0, src.to_glib_none().0);
         res
@@ -1198,21 +1197,25 @@ mod tests {
         {
             let tags = tags.get_mut().unwrap();
             assert!(tags
-                .add_generic(&TAG_TITLE, "some title", TagMergeMode::Append)
+                .add_generic(Title::TAG_NAME, "some title", TagMergeMode::Append)
                 .is_ok());
             assert!(tags
-                .add_generic(&TAG_TITLE, "second title", TagMergeMode::Append)
+                .add_generic(Title::TAG_NAME, "second title", TagMergeMode::Append)
                 .is_ok());
             assert!(tags
-                .add_generic(&TAG_DURATION, ClockTime::SECOND * 120, TagMergeMode::Append)
+                .add_generic(
+                    Duration::TAG_NAME,
+                    ClockTime::SECOND * 120,
+                    TagMergeMode::Append
+                )
                 .is_ok());
             assert!(tags
-                .add_generic(&TAG_TITLE, "third title", TagMergeMode::Append)
+                .add_generic(Title::TAG_NAME, "third title", TagMergeMode::Append)
                 .is_ok());
 
             assert_eq!(
                 tags.add_generic(
-                    &TAG_IMAGE,
+                    Image::TAG_NAME,
                     "`&[str] instead of `Sample`",
                     TagMergeMode::Append
                 ),
@@ -1221,35 +1224,35 @@ mod tests {
         }
 
         assert_eq!(
-            tags.index_generic(&TAG_TITLE, 0).unwrap().get(),
+            tags.index_generic(Title::TAG_NAME, 0).unwrap().get(),
             Ok(Some("some title"))
         );
         assert_eq!(
-            tags.index_generic(&TAG_TITLE, 1).unwrap().get(),
+            tags.index_generic(Title::TAG_NAME, 1).unwrap().get(),
             Ok(Some("second title"))
         );
         assert_eq!(
-            tags.index_generic(&TAG_DURATION, 0).unwrap().get(),
+            tags.index_generic(Duration::TAG_NAME, 0).unwrap().get(),
             Ok(Some(ClockTime::SECOND * 120))
         );
         assert_eq!(
-            tags.index_generic(&TAG_TITLE, 2).unwrap().get(),
+            tags.index_generic(Title::TAG_NAME, 2).unwrap().get(),
             Ok(Some("third title"))
         );
 
         assert_eq!(
-            tags.generic(&TAG_TITLE).unwrap().get(),
+            tags.generic(Title::TAG_NAME).unwrap().get(),
             Ok(Some("some title, second title, third title"))
         );
 
         assert_eq!(tags.n_tags(), 2);
-        assert_eq!(tags.nth_tag_name(0), Some(*TAG_TITLE));
-        assert_eq!(tags.size_by_name(&TAG_TITLE), 3);
-        assert_eq!(tags.nth_tag_name(1), Some(*TAG_DURATION));
-        assert_eq!(tags.size_by_name(&TAG_DURATION), 1);
+        assert_eq!(tags.nth_tag_name(0), Some(Title::TAG_NAME));
+        assert_eq!(tags.size_by_name(Title::TAG_NAME), 3);
+        assert_eq!(tags.nth_tag_name(1), Some(Duration::TAG_NAME));
+        assert_eq!(tags.size_by_name(Duration::TAG_NAME), 1);
 
         // GenericTagIter
-        let mut title_iter = tags.iter_tag_generic(&TAG_TITLE);
+        let mut title_iter = tags.iter_tag_generic(Title::TAG_NAME);
         assert_eq!(title_iter.size_hint(), (3, Some(3)));
         let first_title = title_iter.next().unwrap();
         assert_eq!(first_title.get(), Ok(Some("some title")));
@@ -1264,7 +1267,7 @@ mod tests {
         assert_eq!(tag_list_iter.size_hint(), (2, Some(2)));
 
         let (tag_name, mut tag_iter) = tag_list_iter.next().unwrap();
-        assert_eq!(tag_name, *TAG_TITLE);
+        assert_eq!(tag_name, Title::TAG_NAME);
         let first_title = tag_iter.next().unwrap();
         assert_eq!(first_title.get(), Ok(Some("some title")));
         let second_title = tag_iter.next().unwrap();
@@ -1274,7 +1277,7 @@ mod tests {
         assert!(tag_iter.next().is_none());
 
         let (tag_name, mut tag_iter) = tag_list_iter.next().unwrap();
-        assert_eq!(tag_name, *TAG_DURATION);
+        assert_eq!(tag_name, Duration::TAG_NAME);
         let first_duration = tag_iter.next().unwrap();
         assert_eq!(first_duration.get(), Ok(Some(ClockTime::SECOND * 120)));
         assert!(tag_iter.next().is_none());
@@ -1284,14 +1287,14 @@ mod tests {
         assert_eq!(tag_list_iter.size_hint(), (2, Some(2)));
 
         let (tag_name, tag_value) = tag_list_iter.next().unwrap();
-        assert_eq!(tag_name, *TAG_TITLE);
+        assert_eq!(tag_name, Title::TAG_NAME);
         assert_eq!(
             tag_value.get(),
             Ok(Some("some title, second title, third title"))
         );
 
         let (tag_name, tag_value) = tag_list_iter.next().unwrap();
-        assert_eq!(tag_name, *TAG_DURATION);
+        assert_eq!(tag_name, Duration::TAG_NAME);
         assert_eq!(tag_value.get(), Ok(Some(ClockTime::SECOND * 120)));
         assert!(tag_iter.next().is_none());
     }
@@ -1304,15 +1307,14 @@ mod tests {
 
         impl<'a> Tag<'a> for MyCustomTag {
             type TagType = &'a str;
-            fn tag_name<'b>() -> &'b str {
-                "my-custom-tag"
-            }
+            const TAG_NAME: &'static glib::GStr = glib::gstr!("my-custom-tag");
         }
 
-        impl<'a> CustomTag<'a> for MyCustomTag {
+        impl CustomTag<'_> for MyCustomTag {
             const FLAG: crate::TagFlag = crate::TagFlag::Meta;
-            const NICK: &'static str = "my custom tag";
-            const DESCRIPTION: &'static str = "My own custom tag type for testing";
+            const NICK: &'static glib::GStr = glib::gstr!("my custom tag");
+            const DESCRIPTION: &'static glib::GStr =
+                glib::gstr!("My own custom tag type for testing");
 
             fn merge_func(src: &Value) -> Value {
                 skip_assert_initialized!();
@@ -1322,17 +1324,17 @@ mod tests {
 
         register::<MyCustomTag>();
 
-        assert!(tag_exists(MyCustomTag::tag_name()));
+        assert!(tag_exists(MyCustomTag::TAG_NAME));
         assert_eq!(
-            tag_get_type(MyCustomTag::tag_name()),
+            tag_get_type(MyCustomTag::TAG_NAME),
             <MyCustomTag as Tag>::TagType::static_type()
         );
-        assert_eq!(tag_get_nick(MyCustomTag::tag_name()), MyCustomTag::NICK);
+        assert_eq!(tag_get_nick(MyCustomTag::TAG_NAME), MyCustomTag::NICK);
         assert_eq!(
-            tag_get_description(MyCustomTag::tag_name()),
+            tag_get_description(MyCustomTag::TAG_NAME),
             Some(MyCustomTag::DESCRIPTION)
         );
-        assert_eq!(tag_get_flag(MyCustomTag::tag_name()), MyCustomTag::FLAG);
+        assert_eq!(tag_get_flag(MyCustomTag::TAG_NAME), MyCustomTag::FLAG);
 
         let mut tags = TagList::new();
         {
@@ -1357,7 +1359,7 @@ mod tests {
     fn test_display() {
         crate::init().unwrap();
 
-        format!("{}", TagList::new());
+        let _ = format!("{}", TagList::new());
     }
 
     #[test]
@@ -1365,14 +1367,14 @@ mod tests {
         crate::init().unwrap();
 
         let mut tags = TagList::new();
-        assert_eq!(format!("{:?}", tags), "TagList");
+        assert_eq!(format!("{tags:?}"), "TagList");
         {
             let tags = tags.get_mut().unwrap();
             tags.add::<Title>(&"some title", TagMergeMode::Append);
             tags.add::<Duration>(&(ClockTime::SECOND * 120), TagMergeMode::Append);
         }
         assert_eq!(
-            format!("{:?}", tags),
+            format!("{tags:?}"),
             "TagList { title: (gchararray) \"some title\", duration: (guint64) 120000000000 }"
         );
     }
